@@ -8,9 +8,9 @@ import numpy as np
 
 
 from allel.compat import string_types, integer_types, range
-from allel.model.ndarray import recarray_to_html_str, recarray_display, \
-    subset as _ndarray_subset
 from allel.chunked import util as _util
+from allel.abc import ArrayWrapper, DisplayAsTable
+from allel.model.ndarray import subset as _numpy_subset, NumpyRecArrayWrapper
 
 
 def store(data, arr, start=0, stop=None, offset=0, blen=None):
@@ -53,7 +53,7 @@ def copy(data, start=0, stop=None, blen=None, storage=None, create='array',
     out = None
     for i in range(start, stop, blen):
         j = min(i+blen, stop)
-        block = np.asanyarray(data[i:j])
+        block = data[i:j]
         if out is None:
             out = getattr(storage, create)(block, expectedlen=length, **kwargs)
         else:
@@ -82,7 +82,7 @@ def copy_table(tbl, start=0, stop=None, blen=None, storage=None,
     out = None
     for i in range(start, stop, blen):
         j = min(i+blen, stop)
-        res = [np.asanyarray(c[i:j]) for c in columns]
+        res = [c[i:j] for c in columns]
         if out is None:
             out = getattr(storage, create)(res, names=names,
                                            expectedlen=length, **kwargs)
@@ -92,7 +92,7 @@ def copy_table(tbl, start=0, stop=None, blen=None, storage=None,
     return out
 
 
-def apply(data, f, blen=None, storage=None, create='array', **kwargs):
+def map_blocks(data, f, blen=None, storage=None, create='array', **kwargs):
     """Apply function `f` block-wise over `data`."""
 
     # setup
@@ -114,9 +114,9 @@ def apply(data, f, blen=None, storage=None, create='array', **kwargs):
 
         # obtain blocks
         if isinstance(data, tuple):
-            blocks = [np.asanyarray(d[i:j]) for d in data]
+            blocks = [d[i:j] for d in data]
         else:
-            blocks = [np.asanyarray(data[i:j])]
+            blocks = [data[i:j]]
 
         # map
         res = f(*blocks)
@@ -154,7 +154,7 @@ def reduce_axis(data, reducer, block_reducer, mapper=None, axis=None,
         out = None
         for i in range(0, length, blen):
             j = min(i+blen, length)
-            block = np.asanyarray(data[i:j])
+            block = data[i:j]
             if mapper:
                 block = mapper(block)
             res = reducer(block, axis=axis)
@@ -174,7 +174,7 @@ def reduce_axis(data, reducer, block_reducer, mapper=None, axis=None,
         out = None
         for i in range(0, length, blen):
             j = min(i+blen, length)
-            block = np.asanyarray(data[i:j])
+            block = data[i:j]
             if mapper:
                 block = mapper(block)
             r = reducer(block, axis=axis)
@@ -218,11 +218,14 @@ def count_nonzero(data, mapper=None, blen=None, storage=None,
                        blen=blen, storage=storage, create=create, **kwargs)
 
 
-def compress(data, condition, axis=0, blen=None, storage=None,
-             create='array', **kwargs):
+def compress(condition, data, axis=0, out=None, blen=None, storage=None, create='array',
+             **kwargs):
     """Return selected slices of an array along given axis."""
 
     # setup
+    if out is not None:
+        # argument is only there for numpy API compatibility
+        raise NotImplementedError('out argument is not supported')
     storage = _util.get_storage(storage)
     blen = _util.get_blen_array(data, blen)
     length = len(data)
@@ -235,14 +238,13 @@ def compress(data, condition, axis=0, blen=None, storage=None,
         out = None
         for i in range(0, length, blen):
             j = min(i+blen, length)
-            bcond = np.asanyarray(condition[i:j])
+            bcond = np.asarray(condition[i:j])
             # don't access any data unless we have to
             if np.any(bcond):
-                block = np.asanyarray(data[i:j])
+                block = np.asarray(data[i:j])
                 res = np.compress(bcond, block, axis=0)
                 if out is None:
-                    out = getattr(storage, create)(res, expectedlen=nnz,
-                                                   **kwargs)
+                    out = getattr(storage, create)(res, expectedlen=nnz, **kwargs)
                 else:
                     out.append(res)
         return out
@@ -254,7 +256,7 @@ def compress(data, condition, axis=0, blen=None, storage=None,
         condition = np.asanyarray(condition)
         for i in range(0, length, blen):
             j = min(i+blen, length)
-            block = np.asanyarray(data[i:j])
+            block = np.asarray(data[i:j])
             res = np.compress(condition, block, axis=1)
             if out is None:
                 out = getattr(storage, create)(res, expectedlen=length,
@@ -268,11 +270,14 @@ def compress(data, condition, axis=0, blen=None, storage=None,
         raise NotImplementedError('axis not supported: %s' % axis)
 
 
-def take(data, indices, axis=0, blen=None, storage=None,
+def take(data, indices, axis=0, out=None, mode='raise', blen=None, storage=None,
          create='array', **kwargs):
     """Take elements from an array along an axis."""
 
     # setup
+    if out is not None:
+        # argument is only there for numpy API compatibility
+        raise NotImplementedError('out argument is not supported')
     length = len(data)
 
     if axis == 0:
@@ -287,7 +292,7 @@ def take(data, indices, axis=0, blen=None, storage=None,
         # implement via compress()
         condition = np.zeros((length,), dtype=bool)
         condition[indices] = True
-        return compress(data, condition, axis=0, blen=blen, storage=storage,
+        return compress(condition, data, axis=0, blen=blen, storage=storage,
                         create=create, **kwargs)
 
     elif axis == 1:
@@ -300,8 +305,8 @@ def take(data, indices, axis=0, blen=None, storage=None,
         out = None
         for i in range(0, length, blen):
             j = min(i+blen, length)
-            block = np.asanyarray(data[i:j])
-            res = np.take(block, indices, axis=1)
+            block = data[i:j]
+            res = np.take(block, indices, axis=1, mode=mode)
             if out is None:
                 out = getattr(storage, create)(res, expectedlen=length,
                                                **kwargs)
@@ -313,11 +318,16 @@ def take(data, indices, axis=0, blen=None, storage=None,
         raise NotImplementedError('axis not supported: %s' % axis)
 
 
-def compress_table(tbl, condition, blen=None, storage=None, create='table',
-                   **kwargs):
+def compress_table(condition, tbl, axis=None, out=None, blen=None, storage=None,
+                   create='table', **kwargs):
     """Return selected rows of a table."""
 
     # setup
+    if axis is not None and axis != 0:
+        raise NotImplementedError('only axis 0 is supported')
+    if out is not None:
+        # argument is only there for numpy API compatibility
+        raise NotImplementedError('out argument is not supported')
     storage = _util.get_storage(storage)
     names, columns = _util.check_table_like(tbl)
     blen = _util.get_blen_table(tbl, blen)
@@ -329,10 +339,10 @@ def compress_table(tbl, condition, blen=None, storage=None, create='table',
     out = None
     for i in range(0, length, blen):
         j = min(i+blen, length)
-        bcond = np.asanyarray(condition[i:j])
+        bcond = condition[i:j]
         # don't access any data unless we have to
         if np.any(bcond):
-            bcolumns = [np.asanyarray(c[i:j]) for c in columns]
+            bcolumns = [c[i:j] for c in columns]
             res = [np.compress(bcond, c, axis=0) for c in bcolumns]
             if out is None:
                 out = getattr(storage, create)(res, names=names,
@@ -342,11 +352,18 @@ def compress_table(tbl, condition, blen=None, storage=None, create='table',
     return out
 
 
-def take_table(tbl, indices, blen=None, storage=None, create='table',
-               **kwargs):
+def take_table(tbl, indices, axis=None, out=None, mode='raise', blen=None, storage=None,
+               create='table', **kwargs):
     """Return selected rows of a table."""
 
     # setup
+    if axis is not None and axis != 0:
+        raise NotImplementedError('only axis 0 is supported')
+    if out is not None:
+        # argument is only there for numpy API compatibility
+        raise NotImplementedError('out argument is not supported')
+    if mode is not None and mode != 'raise':
+        raise NotImplementedError('only mode=raise is supported')
     names, columns = _util.check_table_like(tbl)
     length = len(columns[0])
 
@@ -360,7 +377,7 @@ def take_table(tbl, indices, blen=None, storage=None, create='table',
     # implement via compress()
     condition = np.zeros((length,), dtype=bool)
     condition[indices] = True
-    return compress_table(tbl, condition, blen=blen, storage=storage,
+    return compress_table(condition, tbl, blen=blen, storage=storage,
                           create=create, **kwargs)
 
 
@@ -368,6 +385,8 @@ def subset(data, sel0=None, sel1=None, blen=None, storage=None, create='array',
            **kwargs):
     """Return selected rows and columns of an array."""
 
+    # TODO refactor sel0 and sel1 normalization with ndarray.subset
+
     # setup
     storage = _util.get_storage(storage)
     blen = _util.get_blen_array(data, blen)
@@ -387,13 +406,13 @@ def subset(data, sel0=None, sel1=None, blen=None, storage=None, create='array',
     # ensure indices for dim 1
     if sel1 is not None and sel1.dtype.kind == 'b':
         # assume boolean condition, convert to indices
-        sel1 = np.nonzero(sel1)[0]
+        sel1, = np.nonzero(sel1)
 
     # shortcuts
     if sel0 is None and sel1 is None:
         return copy(data, blen=blen, storage=storage, create=create, **kwargs)
     elif sel1 is None:
-        return compress(data, sel0, axis=0, blen=blen, storage=storage,
+        return compress(sel0, data, axis=0, blen=blen, storage=storage,
                         create=create, **kwargs)
     elif sel0 is None:
         return take(data, sel1, axis=1, blen=blen, storage=storage,
@@ -407,8 +426,8 @@ def subset(data, sel0=None, sel1=None, blen=None, storage=None, create='array',
         bsel0 = sel0[i:j]
         # don't access data unless we have to
         if np.any(bsel0):
-            block = np.asanyarray(data[i:j])
-            res = _ndarray_subset(block, bsel0, sel1)
+            block = data[i:j]
+            res = _numpy_subset(block, bsel0, sel1)
             if out is None:
                 out = getattr(storage, create)(res, expectedlen=sel0_nnz,
                                                **kwargs)
@@ -418,48 +437,7 @@ def subset(data, sel0=None, sel1=None, blen=None, storage=None, create='array',
     return out
 
 
-def hstack(tup, blen=None, storage=None, create='array', **kwargs):
-    """Stack arrays in sequence horizontally (column wise)."""
-
-    # setup
-    if not isinstance(tup, (tuple, list)):
-        raise ValueError('expected tuple or list, found %r' % tup)
-    if len(tup) < 2:
-        raise ValueError('expected two or more arrays to stack')
-
-    def f(*blocks):
-        return np.hstack(blocks)
-
-    return apply(tup, f, blen=blen, storage=storage, create=create, **kwargs)
-
-
-def vstack(tup, blen=None, storage=None, create='array', **kwargs):
-    """Stack arrays in sequence vertically (row wise)."""
-
-    # setup
-    storage = _util.get_storage(storage)
-    if not isinstance(tup, (tuple, list)):
-        raise ValueError('expected tuple or list, found %r' % tup)
-    if len(tup) < 2:
-        raise ValueError('expected two or more arrays to stack')
-
-    # build output
-    expectedlen = sum(len(a) for a in tup)
-    out = None
-    for a in tup:
-        ablen = _util.get_blen_array(a, blen)
-        for i in range(0, len(a), ablen):
-            j = min(i+ablen, len(a))
-            block = np.asanyarray(a[i:j])
-            if out is None:
-                out = getattr(storage, create)(block, expectedlen=expectedlen,
-                                               **kwargs)
-            else:
-                out.append(block)
-    return out
-
-
-def vstack_table(tup, blen=None, storage=None, create='table', **kwargs):
+def concatenate_table(tup, blen=None, storage=None, create='table', **kwargs):
     """Stack tables in sequence vertically (row-wise)."""
 
     # setup
@@ -479,7 +457,7 @@ def vstack_table(tup, blen=None, storage=None, create='table', **kwargs):
         tlen = len(tcolumns[0])
         for i in range(0, tlen, tblen):
             j = min(i+tblen, tlen)
-            bcolumns = [np.asanyarray(c[i:j]) for c in tcolumns]
+            bcolumns = [c[i:j] for c in tcolumns]
             if out is None:
                 out = getattr(storage, create)(bcolumns, names=tnames,
                                                expectedlen=expectedlen,
@@ -489,6 +467,41 @@ def vstack_table(tup, blen=None, storage=None, create='table', **kwargs):
     return out
 
 
+def concatenate(tup, axis=0, blen=None, storage=None, create='array', **kwargs):
+    """Concatenate arrays."""
+
+    # setup
+    storage = _util.get_storage(storage)
+    if not isinstance(tup, (tuple, list)):
+        raise ValueError('expected tuple or list, found %r' % tup)
+    if len(tup) < 2:
+        raise ValueError('expected two or more arrays')
+
+    if axis == 0:
+
+        # build output
+        expectedlen = sum(len(a) for a in tup)
+        out = None
+        for a in tup:
+            ablen = _util.get_blen_array(a, blen)
+            for i in range(0, len(a), ablen):
+                j = min(i+ablen, len(a))
+                block = a[i:j]
+                if out is None:
+                    out = getattr(storage, create)(block, expectedlen=expectedlen, **kwargs)
+                else:
+                    out.append(block)
+
+    else:
+
+        def f(*blocks):
+            return np.concatenate(blocks, axis=axis)
+
+        out = map_blocks(tup, f, blen=blen, storage=storage, create=create, **kwargs)
+
+    return out
+
+
 def binary_op(data, op, other, blen=None, storage=None, create='array',
               **kwargs):
     """Compute a binary operation block-wise over `data`."""
@@ -500,14 +513,13 @@ def binary_op(data, op, other, blen=None, storage=None, create='array',
     if np.isscalar(other):
         def f(block):
             return op(block, other)
-        return apply(data, f, blen=blen, storage=storage, create=create,
-                     **kwargs)
+        return map_blocks(data, f, blen=blen, storage=storage, create=create, **kwargs)
 
     elif len(data) == len(other):
         def f(a, b):
             return op(a, b)
-        return apply((data, other), f, blen=blen, storage=storage,
-                     create=create, **kwargs)
+        return map_blocks((data, other), f, blen=blen, storage=storage, create=create,
+                          **kwargs)
 
     else:
         raise NotImplementedError('argument type not supported')
@@ -574,7 +586,7 @@ def eval_table(tbl, expression, vm='python', blen=None, storage=None,
     return out
 
 
-class ChunkedArray(object):
+class ChunkedArrayWrapper(ArrayWrapper):
     """Wrapper class for chunked array-like data.
 
     Parameters
@@ -587,74 +599,56 @@ class ChunkedArray(object):
 
     def __init__(self, data):
         data = _util.ensure_array_like(data)
-        self.data = data
-
-    def __getitem__(self, *args):
-        return self.data.__getitem__(*args)
-
-    def __setitem__(self, key, value):
-        return self.data.__setitem__(key, value)
-
-    def __getattr__(self, item):
-        return getattr(self.data, item)
-
-    def __array__(self, *args):
-        return self[:]
-
-    def __repr__(self):
-        r = '%s(' % type(self).__name__
-        r += '%s' % str(self.shape)
-        r += ', %s' % str(self.dtype)
-        if self.chunks is not None:
-            r += ', chunks=%s' % str(self.chunks)
-        r += ')'
-        if self.nbytes:
-            r += '\n  nbytes: %s;' % _util.human_readable_size(self.nbytes)
-            if self.cbytes:
-                r += ' cbytes: %s;' % _util.human_readable_size(self.cbytes)
-            if self.cratio:
-                r += ' cratio: %.1f;' % self.cratio
-        if self.compression:
-            r += '\n  compression: %s;' % self.compression
-            if self.compression_opts is not None:
-                r += ' compression_opts: %s;' % self.compression_opts
-        r += '\n  data: %s.%s' % (type(self.data).__module__,
-                                  type(self.data).__name__)
-        return r
-
-    def __str__(self):
-        return str(self.data)
-
-    def __len__(self):
-        return len(self.data)
+        super(ChunkedArrayWrapper, self).__init__(data)
 
     @property
-    def ndim(self):
-        return len(self.shape)
+    def caption(self):
+        r = '<%s' % type(self).__name__
+        r += ' shape=%s' % str(self.shape)
+        r += ' dtype=%s' % str(self.dtype)
+        if self.chunks is not None:
+            r += ' chunks=%s' % str(self.chunks)
+        if self.nbytes:
+            r += '\n   nbytes=%s' % _util.human_readable_size(self.nbytes)
+            if self.cbytes:
+                r += ' cbytes=%s' % _util.human_readable_size(self.cbytes)
+            if self.cratio:
+                r += ' cratio=%.1f' % self.cratio
+        if self.compression:
+            r += '\n   compression=%s' % self.compression
+            if self.compression_opts is not None:
+                r += ' compression_opts=%s' % self.compression_opts
+        values_cls = type(self.values)
+        r += '\n   values=%s.%s' % (values_cls.__module__, values_cls.__name__)
+        r += '>'
+        return r
+
+    def __repr__(self):
+        return self.caption
 
     @property
     def nbytes(self):
-        return _util.get_nbytes(self.data)
+        return _util.get_nbytes(self.values)
 
     @property
     def cbytes(self):
-        return _util.get_cbytes(self.data)
+        return _util.get_cbytes(self.values)
 
     @property
     def compression(self):
-        return _util.get_compression(self.data)
+        return _util.get_compression(self.values)
 
     @property
     def compression_opts(self):
-        return _util.get_compression_opts(self.data)
+        return _util.get_compression_opts(self.values)
 
     @property
     def shuffle(self):
-        return _util.get_shuffle(self.data)
+        return _util.get_shuffle(self.values)
 
     @property
     def chunks(self):
-        return _util.get_chunks(self.data)
+        return _util.get_chunks(self.values)
 
     @property
     def cratio(self):
@@ -666,19 +660,13 @@ class ChunkedArray(object):
 
     # outputs from these methods are not wrapped
     store = store
-    reduce_axis = reduce_axis
-    max = amax
-    min = amin
-    sum = asum
     count_nonzero = count_nonzero
 
-    def apply(self, f, blen=None, storage=None, create='array', **kwargs):
-        out = apply(self, f, blen=blen, storage=storage, create=create,
-                    **kwargs)
-        # don't wrap, leave this up to user
-        return out
+    def map_blocks(self, f, blen=None, storage=None, create='array', **kwargs):
+        out = map_blocks(self, f, blen=blen, storage=storage, create=create, **kwargs)
+        return ChunkedArrayWrapper(out)
 
-    def apply_method(self, method_name, kwargs=None, **storage_kwargs):
+    def map_blocks_method(self, method_name, kwargs=None, **storage_kwargs):
         if kwargs is None:
             kwargs = dict()
 
@@ -686,49 +674,21 @@ class ChunkedArray(object):
             method = getattr(block, method_name)
             return method(**kwargs)
 
-        out = self.apply(f, **storage_kwargs)
-        # don't wrap, leave this up to user
+        out = self.map_blocks(f, **storage_kwargs)
         return out
 
     def copy(self, start=0, stop=None, blen=None, storage=None, create='array',
              **kwargs):
         out = copy(self, start=start, stop=stop, blen=blen, storage=storage,
                    create=create, **kwargs)
-        return type(self)(out)
-
-    def compress(self, condition, axis=0, blen=None, storage=None,
-                 create='array', **kwargs):
-        out = compress(self, condition, axis=axis, blen=blen,
-                       storage=storage, create=create, **kwargs)
-        return type(self)(out)
-
-    def take(self, indices, axis=0, blen=None, storage=None,
-             create='array', **kwargs):
-        out = take(self, indices, axis=axis, blen=blen, storage=storage,
-                   create=create, **kwargs)
-        return type(self)(out)
-
-    def subset(self, sel0, sel1, blen=None, storage=None, create='array',
-               **kwargs):
-        out = subset(self, sel0, sel1, blen=blen, storage=storage,
-                     create=create, **kwargs)
-        return type(self)(out)
-
-    def hstack(self, *others, **kwargs):
-        tup = (self,) + others
-        out = hstack(tup, **kwargs)
-        return type(self)(out)
-
-    def vstack(self, *others, **kwargs):
-        tup = (self,) + others
-        out = vstack(tup, **kwargs)
+        # can always wrap this as sub-class
         return type(self)(out)
 
     def binary_op(self, op, other, blen=None, storage=None, create='array',
                   **kwargs):
         out = binary_op(self, op, other, blen=blen, storage=storage,
                         create=create, **kwargs)
-        return ChunkedArray(out)
+        return ChunkedArrayWrapper(out)
 
     def __eq__(self, other, **kwargs):
         return self.binary_op(operator.eq, other, **kwargs)
@@ -769,8 +729,63 @@ class ChunkedArray(object):
     def __truediv__(self, other, **kwargs):
         return self.binary_op(operator.truediv, other, **kwargs)
 
+    def __and__(self, other, **kwargs):
+        return self.binary_op(operator.and_, other, **kwargs)
 
-class ChunkedTable(object):
+    def __lshift__(self, other, **kwargs):
+        return self.binary_op(operator.lshift, other, **kwargs)
+
+    def __or__(self, other, **kwargs):
+        return self.binary_op(operator.or_, other, **kwargs)
+
+    def __rshift__(self, other, **kwargs):
+        return self.binary_op(operator.rshift, other, **kwargs)
+
+    def __xor__(self, other, **kwargs):
+        return self.binary_op(operator.xor, other, **kwargs)
+
+    def compress(self, condition, axis=0, out=None, **kwargs):
+        out = compress(condition, self.values, axis=axis, out=out, **kwargs)
+        return ChunkedArrayWrapper(out)
+
+    def take(self, indices, axis=0, out=None, **kwargs):
+        out = take(self.values, indices, axis=axis, out=out, **kwargs)
+        return ChunkedArrayWrapper(out)
+
+    def subset(self, sel0=None, sel1=None, **kwargs):
+        out = subset(self.values, sel0, sel1, **kwargs)
+        return ChunkedArrayWrapper(out)
+
+    def concatenate(self, others, axis=0, **kwargs):
+        if not isinstance(others, (tuple, list)):
+            others = others,
+        tup = (self,) + tuple(others)
+        out = concatenate(tup, axis=axis, **kwargs)
+        return ChunkedArrayWrapper(out)
+
+    def max(self, axis=None, **kwargs):
+        out = amax(self, axis=axis, **kwargs)
+        if np.isscalar(out):
+            return out
+        else:
+            return ChunkedArrayWrapper(out)
+
+    def min(self, axis=None, **kwargs):
+        out = amin(self, axis=axis, **kwargs)
+        if np.isscalar(out):
+            return out
+        else:
+            return ChunkedArrayWrapper(out)
+
+    def sum(self, axis=None, **kwargs):
+        out = asum(self, axis=axis, **kwargs)
+        if np.isscalar(out):
+            return out
+        else:
+            return ChunkedArrayWrapper(out)
+
+
+class ChunkedTableWrapper(DisplayAsTable):
     """Wrapper class for chunked table-like data.
 
     Parameters
@@ -784,25 +799,35 @@ class ChunkedTable(object):
 
     """
 
-    view_cls = np.recarray
+    array_cls = NumpyRecArrayWrapper
 
+    # noinspection PyMissingConstructor
     def __init__(self, data, names=None):
         names, columns = _util.check_table_like(data, names=names)
-        self.data = data
-        self.names = names
-        self.columns = columns
+        # skip super-class constructor because we are more flexible about type of values here
+        self._values = data
+        self._names = names
+        self._columns = columns
         self.rowcls = namedtuple('row', names)
 
+    @property
+    def names(self):
+        return self._names
+
+    @property
+    def columns(self):
+        return self._columns
+
     def __getitem__(self, item):
 
         if isinstance(item, string_types):
             # item is column name, return column
-            idx = self.names.index(item)
-            return ChunkedArray(self.columns[idx])
+            idx = self._names.index(item)
+            return ChunkedArrayWrapper(self._columns[idx])
 
         elif isinstance(item, integer_types):
             # item is row index, return row
-            return self.rowcls(*(col[item] for col in self.columns))
+            return self.rowcls(*(col[item] for col in self._columns))
 
         elif isinstance(item, slice):
             # item is row slice, return numpy recarray
@@ -814,60 +839,56 @@ class ChunkedTable(object):
             step = 1 if item.step is None else item.step
             outshape = (stop - start) // step
             out = np.empty(outshape, dtype=self.dtype)
-            for n, c in zip(self.names, self.columns):
+            for n, c in zip(self._names, self._columns):
                 out[n] = c[start:stop:step]
-            return out.view(self.view_cls)
+            out = out.view(np.recarray)
+            if self.array_cls is not None:
+                out = self.array_cls(out)
+            return out
 
         elif isinstance(item, (list, tuple)) and \
                 all(isinstance(i, string_types) for i in item):
             # item is sequence of column names, return table
-            columns = [self.columns[self.names.index(n)] for n in item]
+            columns = [self._columns[self._names.index(n)] for n in item]
             return type(self)(columns, names=item)
 
         else:
-            raise NotImplementedError('item not suppored: %r' % item)
+            raise IndexError('item not supported for indexing: %s' % repr(item))
 
-    def __array__(self):
-        return np.asarray(self[:])
+    def __array__(self, *args):
+        a = np.asanyarray(self[:])
+        if args:
+            a = a.astype(args[0])
+        return a
 
     def __getattr__(self, item):
-        if item in self.names:
-            idx = self.names.index(item)
-            return ChunkedArray(self.columns[idx])
+        if item in self._names:
+            idx = self._names.index(item)
+            return ChunkedArrayWrapper(self._columns[idx])
         else:
-            raise AttributeError(item)
+            return super(ChunkedTableWrapper, self).__getattr__(item)
 
-    def __repr__(self):
-        r = '%s(' % type(self).__name__
-        r += '%s' % len(self)
-        r += ')'
+    @property
+    def caption(self):
+        r = '<%s' % type(self).__name__
+        r += ' shape=%s' % str(self.shape)
+        r += ' dtype=%s' % str(self.dtype)
         if self.nbytes:
-            r += '\n  nbytes: %s;' % _util.human_readable_size(self.nbytes)
+            r += '\n   nbytes=%s' % _util.human_readable_size(self.nbytes)
             if self.cbytes:
-                r += ' cbytes: %s;' % _util.human_readable_size(self.cbytes)
+                r += ' cbytes=%s' % _util.human_readable_size(self.cbytes)
             if self.cratio:
-                r += ' cratio: %.1f;' % self.cratio
-        r += '\n  data: %s.%s' % (type(self.data).__module__,
-                                  type(self.data).__name__)
+                r += ' cratio=%.1f' % self.cratio
+        values_cls = type(self.values)
+        r += '\n   values=%s.%s' % (values_cls.__module__, values_cls.__name__)
+        r += '>'
         return r
 
+    def __repr__(self):
+        return self.caption
+
     def __len__(self):
-        return len(self.columns[0])
-
-    def _repr_html_(self):
-        caption = repr(self)
-        ra = self[:6]
-        return recarray_to_html_str(ra, limit=5, caption=caption)
-
-    def display(self, limit=5, **kwargs):
-        kwargs.setdefault('caption', repr(self))
-        if limit is None:
-            limit = len(self)
-        ra = self[:limit+1]
-        return recarray_display(ra, limit=limit, **kwargs)
-
-    def displayall(self, **kwargs):
-        return self.display(limit=None, **kwargs)
+        return len(self._columns[0])
 
     @property
     def shape(self):
@@ -880,7 +901,7 @@ class ChunkedTable(object):
     @property
     def dtype(self):
         l = []
-        for n, c in zip(self.names, self.columns):
+        for n, c in zip(self._names, self._columns):
             # need to account for multidimensional columns
             t = (n, c.dtype) if len(c.shape) == 1 else \
                 (n, c.dtype, c.shape[1:])
@@ -889,14 +910,14 @@ class ChunkedTable(object):
 
     @property
     def nbytes(self):
-        cols_nbytes = [_util.get_nbytes(c) for c in self.columns]
+        cols_nbytes = [_util.get_nbytes(c) for c in self._columns]
         if all(cols_nbytes):
             return sum(cols_nbytes)
         return None
 
     @property
     def cbytes(self):
-        cols_cbytes = [_util.get_cbytes(c) for c in self.columns]
+        cols_cbytes = [_util.get_cbytes(c) for c in self._columns]
         if all(cols_cbytes):
             return sum(cols_cbytes)
         return None
@@ -913,35 +934,29 @@ class ChunkedTable(object):
              create='table', **kwargs):
         out = copy_table(self, start=start, stop=stop, blen=blen,
                          storage=storage, create=create, **kwargs)
-        return type(self)(out, names=self.names)
-
-    def compress(self, condition, blen=None, storage=None, create='table',
-                 **kwargs):
-        out = compress_table(self, condition, blen=blen, storage=storage,
-                             create=create, **kwargs)
-        return type(self)(out, names=self.names)
-
-    def take(self, indices, blen=None, storage=None, create='table', **kwargs):
-        out = take_table(self, indices, blen=blen, storage=storage,
-                         create=create, **kwargs)
-        return type(self)(out, names=self.names)
-
-    def vstack(self, *others, **kwargs):
-        tup = (self,) + others
-        out = vstack_table(tup, **kwargs)
-        return type(self)(out, names=self.names)
+        # can always wrap this
+        return type(self)(out, names=self._names)
 
     def eval(self, expression, **kwargs):
         out = eval_table(self, expression, **kwargs)
-        return ChunkedArray(out)
+        return ChunkedArrayWrapper(out)
 
-    def query(self, expression, vm='python', blen=None, storage=None,
-              create='table', vm_kwargs=None, **kwargs):
-        condition = self.eval(expression, vm=vm, blen=blen, storage=storage,
-                              create='array', vm_kwargs=vm_kwargs)
-        out = self.compress(condition, blen=blen, storage=storage,
-                            create=create, **kwargs)
+    def query(self, expression, vm='python', blen=None, storage=None, create='table',
+              vm_kwargs=None, **kwargs):
+        condition = self.eval(expression, vm=vm, blen=blen, storage=storage, create='array',
+                              vm_kwargs=vm_kwargs)
+        out = self.compress(condition, blen=blen, storage=storage, create=create, **kwargs)
+        # should already be wrapped
         return out
 
-    # TODO addcol (and __setitem__?)
-    # TODO delcol (and __delitem__?)
+    def compress(self, condition, axis=None, out=None, blen=None, storage=None, create='table',
+                 **kwargs):
+        out = compress_table(condition, self, axis=axis, out=out, blen=blen, storage=storage,
+                             create=create, **kwargs)
+        return type(self)(out)
+
+    def take(self, indices, axis=None, out=None, mode='raise', blen=None, storage=None,
+             create='table', **kwargs):
+        out = take_table(self, indices, axis=axis, out=out, mode=mode, blen=blen,
+                         storage=storage, create=create, **kwargs)
+        return type(self)(out)
