@@ -7,10 +7,12 @@ from multiprocessing.pool import ThreadPool
 import numpy as np
 
 
-from allel.util import asarray_ndim, check_dim0_aligned
+from allel.util import asarray_ndim, check_dim0_aligned, check_ndim
 from allel.model.ndarray import HaplotypeArray
 from allel.stats.window import moving_statistic, index_windows
 from allel.stats.diversity import moving_tajima_d
+from allel.opt.stats import pairwise_shared_prefix_lengths_int8, paint_shared_prefixes_int8, \
+    ihh01_scan_int8, ihh_scan_int8, nsl01_scan_int8, nsl_scan_int8
 
 
 def ehh_decay(h, truncate=False):
@@ -31,8 +33,6 @@ def ehh_decay(h, truncate=False):
 
     """
 
-    from allel.opt.stats import pairwise_shared_prefix_lengths_int8
-
     # check inputs
     # N.B., ensure int8 so we can use cython optimisation
     h = HaplotypeArray(np.asarray(h, dtype='i1'), copy=False)
@@ -77,8 +77,6 @@ def voight_painting(h):
 
     """
 
-    from allel.opt.stats import paint_shared_prefixes_int8
-
     # check inputs
     # N.B., ensure int8 so we can use cython optimisation
     h = HaplotypeArray(np.asarray(h, dtype='i1'), copy=False)
@@ -386,10 +384,9 @@ def ihs(h, pos, map_pos=None, min_ehh=0.05, min_maf=0.05, include_edges=False,
 
     """
 
-    from allel.opt.stats import ihh01_scan_int8
-
     # check inputs
-    h = HaplotypeArray(np.asarray(h, dtype='i1'))
+    h = np.asarray(h, dtype='i1')
+    check_ndim(h, 2)
     pos = asarray_ndim(pos, 1)
     check_dim0_aligned(h, pos)
 
@@ -397,8 +394,7 @@ def ihs(h, pos, map_pos=None, min_ehh=0.05, min_maf=0.05, include_edges=False,
     gaps = compute_ihh_gaps(pos, map_pos, gap_scale, max_gap, is_accessible)
 
     # setup kwargs
-    kwargs = dict(min_ehh=min_ehh, min_maf=min_maf,
-                  include_edges=include_edges)
+    kwargs = dict(min_ehh=min_ehh, min_maf=min_maf, include_edges=include_edges)
 
     if use_threads and multiprocessing.cpu_count() > 1:
         # run with threads
@@ -506,11 +502,11 @@ def xpehh(h1, h2, pos, map_pos=None, min_ehh=0.05, include_edges=False,
 
     """
 
-    from allel.opt.stats import ihh_scan_int8
-
     # check inputs
-    h1 = HaplotypeArray(np.asarray(h1, dtype='i1'))
-    h2 = HaplotypeArray(np.asarray(h2, dtype='i1'))
+    h1 = np.asarray(h1, dtype='i1')
+    h2 = np.asarray(h2, dtype='i1')
+    check_ndim(h1, 2)
+    check_ndim(h2, 2)
     pos = asarray_ndim(pos, 1)
     check_dim0_aligned(h1, h2, pos)
 
@@ -531,10 +527,8 @@ def xpehh(h1, h2, pos, map_pos=None, min_ehh=0.05, include_edges=False,
         res2_fwd = pool.apply_async(ihh_scan_int8, (h2, gaps), kwargs)
 
         # scan backward
-        res1_rev = pool.apply_async(ihh_scan_int8, (h1[::-1], gaps[::-1]),
-                                    kwargs)
-        res2_rev = pool.apply_async(ihh_scan_int8, (h2[::-1], gaps[::-1]),
-                                    kwargs)
+        res1_rev = pool.apply_async(ihh_scan_int8, (h1[::-1], gaps[::-1]), kwargs)
+        res2_rev = pool.apply_async(ihh_scan_int8, (h2[::-1], gaps[::-1]), kwargs)
 
         # wait for both to finish
         pool.close()
@@ -611,10 +605,9 @@ def nsl(h, use_threads=True):
 
     """
 
-    from allel.opt.stats import nsl01_scan_int8
-
     # check inputs
-    h = HaplotypeArray(np.asarray(h, dtype='i1'))
+    h = np.asarray(h, dtype='i1')
+    check_ndim(h, 2)
 
     # # check there are no invariant sites
     # ac = h.count_alleles()
@@ -677,11 +670,13 @@ def xpnsl(h1, h2, use_threads=True):
         Unstandardized XPNSL scores.
 
     """
-    from allel.opt.stats import nsl_scan_int8
 
     # check inputs
-    h1 = HaplotypeArray(np.asarray(h1, dtype='i1'))
-    h2 = HaplotypeArray(np.asarray(h2, dtype='i1'))
+    h1 = np.asarray(h1, dtype='i1')
+    h2 = np.asarray(h2, dtype='i1')
+    check_ndim(h1, 2)
+    check_ndim(h2, 2)
+    check_dim0_aligned(h1, h2)
 
     if use_threads and multiprocessing.cpu_count() > 1:
         # use multiple threads
@@ -1046,7 +1041,7 @@ def plot_moving_haplotype_frequencies(pos, h, size, start=0, stop=None, n=None,
         hr[hr > n] = 0
 
     # compute window start and stop positions
-    windows = moving_statistic(pos, statistic=lambda x: (x[0], x[-1]),
+    windows = moving_statistic(pos, statistic=lambda v: (v[0], v[-1]),
                                size=size, start=start, stop=stop)
 
     # create color map
@@ -1098,9 +1093,9 @@ def moving_delta_tajima_d(ac1, ac2, size, start=0, stop=None, step=None):
 
     """
 
-    D1 = moving_tajima_d(ac1, size=size, start=start, stop=stop, step=step)
-    D2 = moving_tajima_d(ac2, size=size, start=start, stop=stop, step=step)
-    delta = D1 - D2
+    d1 = moving_tajima_d(ac1, size=size, start=start, stop=stop, step=step)
+    d2 = moving_tajima_d(ac2, size=size, start=start, stop=stop, step=step)
+    delta = d1 - d2
     delta_z = (delta - np.mean(delta)) / np.std(delta)
     return delta_z
 
